package agileTraveler.database;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import agileTraveler.exceptions.CityNotFoundException;
import agileTraveler.model.Airport;
import agileTraveler.model.City;
import agileTraveler.utilities.StringUtils;

public class Database {
	
	private Connection connection = null;
	private static Database instance = null;
	
	public static Database getDatabase() throws ClassNotFoundException, SQLException {
		if (instance == null) {
			instance = new Database();
		}
		return instance;
	}
	
	private Database() throws ClassNotFoundException, SQLException {
		Class.forName("org.sqlite.JDBC");
		URI uri;
		try {
			uri = getClass().getResource("/database.sqlite").toURI();
		} catch (URISyntaxException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		String dbPath = new File(uri).getAbsolutePath();
		connection = DriverManager.getConnection("jdbc:sqlite:" + dbPath);
	}

	public List<City> getCities(String namePart) throws SQLException {
		return searchCities(namePart);
	}
	
	
	public String getCitiesAsString (String namePart) throws SQLException {
		List<City> cities = getCities(namePart);
		
		String entry = "";
		for(City city : cities){
			entry += city;
		}
		return entry;
	}

	private List<City> convertCities(ResultSet resultSet) throws SQLException {
		List<City> cities = new ArrayList<City>();
		String lastCityCode = "";
		City city = null;
		while (resultSet.next()) {
			String currentCityCode = resultSet.getString("city_code");
			if (!currentCityCode.equals(lastCityCode)) {
				//we got a new city
				lastCityCode = currentCityCode;
				city = new City();
				city.setCode(currentCityCode);
				city.setName(resultSet.getString("city_name"));
				city.setCountry(resultSet.getString("country"));
				city.setAirports(new ArrayList<Airport>());
				cities.add(city);
			}
			if (resultSet.getString("airport_code") != null) {
				Airport airport = new Airport (resultSet.getString("airport_name"),  resultSet.getString("airport_code"));
				city.getAirports().add(airport);
			}
		}
		return cities;
	}

	private List<City> searchCities(String namePart) throws SQLException {
		List<City> citiesBeginning = searchCitiesBeginningWith(namePart);
		List<City> cities = searchCitiesHaving(namePart, 10 - citiesBeginning.size());
		return mergeCities(citiesBeginning, cities);
	}

	private List<City> mergeCities(List<City> citiesBeginning, List<City> cities) {
		Map<String, City> mapBeginWith = ListToMap(citiesBeginning);
		List<City> doesNotBeginWith = new ArrayList<City>();
		for (City city : cities) {
			if (!mapBeginWith.containsKey(city.getName()))
				doesNotBeginWith.add(city);
		}
		Collections.sort(doesNotBeginWith);
		citiesBeginning.addAll(doesNotBeginWith);
		return citiesBeginning;
	}

	private List<City> searchCitiesHaving(String namePart, int limit) throws SQLException {
		String searchString = "'%" + normalize(namePart) + "%'";
		String sql = "select city_code, city_name, country, airport_name, airport_code " +
				"from CITIES_AND_AIRPORTS " +
				"where (airport_code is null and lower(city_name || ', ' || country) like " + searchString + ") " +
				" or (airport_code is not null" +
					" and lower(airport_name || ' (' || airport_code || '), ' || country) like " + searchString + ")" +
				" order by city_name LIMIT " + limit;
		PreparedStatement pstmt = connection.prepareStatement(sql);
		ResultSet resultSet = pstmt.executeQuery();
		return convertCities(resultSet);
	}

	private List<City> searchCitiesBeginningWith(String namePart) throws SQLException {
		String searchStringBeginning = "'" + normalize(namePart) + "%'";
		String sql = "select city_code, city_name, country, airport_name, airport_code " +
				"from CITIES_AND_AIRPORTS " +
				"where lower(airport_name) like " + searchStringBeginning +
				" or   lower(city_name)    like " + searchStringBeginning +
				" order by city_name LIMIT 10";
		PreparedStatement pstmt = connection.prepareStatement(sql);
		ResultSet resultSet = pstmt.executeQuery();
		return convertCities(resultSet);
	}
	
	private Map<String, City> ListToMap(List<City> cities) {
		Map<String, City> map = new HashMap<String, City>();
		for (City city : cities) {
			map.put(city.getName(), city);
		}
		return map;
	}

	public String normalize(String string) {
		return StringUtils.removeAccents(string).toLowerCase();
	}

	@Override
	public void finalize() throws Throwable {
		try {
			if (connection != null)
				connection.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		super.finalize();
	}

	public String getCityCode(String cityName) throws CityNotFoundException, SQLException {
		String sql = "select code from cities where lower(name) = ? order by code desc limit 1";
		PreparedStatement pstmt = connection.prepareStatement(sql);
		String normalized = normalize(cityName);
		pstmt.setString(1, normalized);
		ResultSet resultSet = pstmt.executeQuery();
		if (!resultSet.next())
			throw new CityNotFoundException("No location code for city " + cityName);
		return resultSet.getString(1);
	}

	public String getCityName(String cityCode) throws SQLException, CityNotFoundException {
		String sql = "select name from cities where code = ? order by name desc limit 1";
		PreparedStatement pstmt = connection.prepareStatement(sql);
		pstmt.setString(1, cityCode);
		ResultSet resultSet = pstmt.executeQuery();
		if (!resultSet.next())
			throw new CityNotFoundException("No location for this code " + cityCode);
		return resultSet.getString(1);
	}
}
