package agileTraveler.database;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.sql.SQLException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.BeforeClass;
import org.junit.Test;

import agileTraveler.model.Airport;
import agileTraveler.model.City;

public class DatabaseTest {
	
	private static Database database;
	
	@BeforeClass
	public static void setUp() throws Exception {
		database = Database.getDatabase();
	}

	@Test
	public void shouldReturnCodeOfAnUniqueCityName() throws Exception {
		List<City> cities = database.getCities("Quixeramobim");
		assertNotNull(cities);
		assertEquals(1, cities.size());
		assertEquals("BRXX0194", cities.get(0).getCode());
		assertEquals("Quixeramobim", cities.get(0).getName());
		assertEquals("Brazil", cities.get(0).getCountry());
	}
	
	@Test
	public void shoulReturnuniqueCityForIncompleteName() throws Exception {
		List<City> cities = database.getCities("Quixera");
		assertNotNull(cities);
		assertEquals(1, cities.size());
		assertEquals("BRXX0194", cities.get(0).getCode());
		assertEquals("Quixeramobim", cities.get(0).getName());
		assertEquals("Brazil", cities.get(0).getCountry());
	}
	
	@Test
	public void shouldReturnCityForAnyPartOfTheName() throws Exception {
		List<City> cities = database.getCities("xeramo");
		assertNotNull(cities);
		assertEquals(1, cities.size());
		assertEquals("BRXX0194", cities.get(0).getCode());
		assertEquals("Quixeramobim", cities.get(0).getName());
		assertEquals("Brazil", cities.get(0).getCountry());
	}
	
	@Test
	public void shoulReturnMultipleCities() throws Exception {
		List<City> cities = database.getCities("Paris");
		assertNotNull(cities);
		assertTrue(cities.size() > 1);
		for (City city : cities)
			assertTrue(city.getName().contains("Paris"));
	}
	
	@Test
	public void shouldReturnEmptyResultForUnknownCity() throws Exception {
		List<City> cities = database.getCities("Unkown city");
		assertNotNull(cities);
		assertTrue(cities.isEmpty());
	}
	
	@Test
	public void shouldReturnAllCityAirports() throws Exception {
		List<City> cities = database.getCities("sÃo pAUlO");
		assertEquals(4, cities.size());
		assertEquals("BRXX0232", cities.get(0).getCode());
		assertEquals("BRXX3518", cities.get(1).getCode());
		assertEquals("BRXX3519", cities.get(2).getCode());
		assertEquals(4, cities.get(0).getAirports().size());
		assertEquals(0, cities.get(1).getAirports().size());
		assertEquals(0, cities.get(2).getAirports().size());
		assertEquals(1, cities.get(3).getAirports().size());
		List<Airport> airs = cities.get(1).getAirports();
		Set<String> codes = new HashSet<String>();
		codes.add("SAO");
		codes.add("CGH");
		codes.add("GRU");
		codes.add("VCP");
		Set<String> names = new HashSet<String>();
		names.add("Sao Paulo - Congonhas");
		names.add("Sao Paulo - Guarulhos");
		names.add("Sao Paulo - Viracopos");
		names.add("Sao Paulo");
		for (Airport air : airs) {
			assertTrue(codes.contains(air.getCode()));
			assertTrue(names.contains(air.getName()));
		}
	}
	
	@Test
	public void shouldReturnAtMost10Cities() throws Exception {
		List<City> cities = database.getCities("a");
		assertTrue(cities.size() <= 10);
	}
	
	@Test
	public void shouldReturnCityNameForCityCode() throws Exception {
		String cityName = database.getCityName("FRXX0076");
		assertEquals("Paris", cityName);
		
		cityName = database.getCityName("BRXX0232");
		assertEquals("Sao Paulo", cityName);
	}
	
	@Test
	public void shouldNotRequireTrailingSpaceToShowRome() throws SQLException {
		List<City> cities = database.getCities("Rome");
		boolean romeFound = false;
		for (City city : cities) {
			if (city.getName().equals("Rome"))
				romeFound = true;
		}
		assertTrue(romeFound);
	}
	
	@Test
	public void shouldFindCityByPart() throws SQLException {
		String found = database.getCitiesAsString("York");
		String template = "York, United Kingdom\n" + 
		"UKXX0162\n" +
		"Yorklyn, United States\n" +
		"USDE0058\n" +
		"Yorkton, Canada\n" +
		"CAXX0558\n" +
		"New York - John F. Kennedy, NY (JFK), United States\n" +
		"USNY0996\n" +
		"New York - LaGuardia, NY (LGA), United States\n" +
		"USNY0996\n" +
		"New York - Newark, NJ (EWR), United States\n" +
		"USNY0996\n" +
		"New York, NY (NYC), United States\n" +
		"USNY0996\n" +
		"North York, Canada\n" +
		"CAXX0330\n" +
		"Richmond (North Yorkshire), United Kingdom\n" +
		"UKXX1791\n";
		assertEquals(template, found);
	}
	
	@Test
	public void shouldFindCityByAiportCode() throws Exception {
		City city = database.getCities("ozz").get(0);
		assertEquals("Ouarzazate", city.getName());
		assertEquals("OZZ", city.getAirports().get(0).getCode());
	}
	
	@Test
	public void shouldFindCityWithCompleteName() throws Exception {
		City city = database.getCities("São Paulo - Congonhas (CGH), Brazil").get(0);
		assertEquals("Sao Paulo", city.getName());
		assertEquals("Brazil", city.getCountry());
		Airport airport = city.getAirports().get(0);
		assertEquals("CGH", airport.getCode());
		assertEquals("Sao Paulo - Congonhas", airport.getName());
	}

	@Test
	public void shouldFindBrazilCities() throws Exception {
		List<City> cities = database.getCities("Brazil");
		for (City city : cities)
			assertEquals("Brazil", city.getCountry());
	}
	
	@Test
	public void shouldFindPinamar() throws Exception {
		City city = database.getCities("Pinamar, Argentin").get(0);
		assertEquals("Pinamar", city.getName());
		assertEquals("Argentina", city.getCountry());
	}
}