package agileTraveler.utilities;

import static agileTraveler.utilities.DateUtils.daysFromTodayISO;
import static java.util.Calendar.APRIL;
import static java.util.Calendar.AUGUST;
import static java.util.Calendar.DAY_OF_MONTH;
import static java.util.Calendar.DECEMBER;
import static java.util.Calendar.FEBRUARY;
import static java.util.Calendar.JANUARY;
import static java.util.Calendar.JULY;
import static java.util.Calendar.JUNE;
import static java.util.Calendar.MARCH;
import static java.util.Calendar.MAY;
import static java.util.Calendar.MONTH;
import static java.util.Calendar.NOVEMBER;
import static java.util.Calendar.OCTOBER;
import static java.util.Calendar.SEPTEMBER;
import static java.util.Calendar.YEAR;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.junit.Test;

public class DateUtilsTest {

	private Calendar addYear(int years) {
		Calendar date = Calendar.getInstance();
		date.setTime(new Date());
		date.add(Calendar.YEAR, years);
		return date;
	}
	

	@Test
	public void testIsValidPeriod() throws Exception {
		Date dateBegin = DateUtils.stringToDate("2010-02-01");
		Date dateEnd = DateUtils.stringToDate("2010-05-15");
		assertTrue(DateUtils.isPeriodValid(dateBegin, dateEnd));
		assertFalse(DateUtils.isPeriodValid(dateEnd, dateBegin));
	}

	@Test
	public void testIsPastDate() throws Exception {
		Date pastDate = DateUtils.stringToDate("2000-01-01");
		assertFalse(DateUtils.isFutureDate(pastDate));

		Calendar futureDate = addYear(1);
		assertTrue(DateUtils.isFutureDate(futureDate.getTime()));
	}

	@Test
	public void shouldReturnNullForEmptyDate() {
		assertNull(DateUtils.stringToDate(null));
		assertNull(DateUtils.stringToDate("   "));
	}

	@Test
	public void shouldReturnValidDate() throws Exception {
		Date date = DateUtils.stringToDate("2010-04-16");
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		assertEquals(16, calendar.get(DAY_OF_MONTH));
		assertEquals(APRIL, calendar.get(MONTH));
		assertEquals(2010, calendar.get(YEAR));

		date = DateUtils.stringToDate("2010-02-28");
		calendar.setTime(date);
		assertEquals(28, calendar.get(DAY_OF_MONTH));
		assertEquals(FEBRUARY, calendar.get(MONTH));
		assertEquals(2010, calendar.get(YEAR));
	}

	@Test
	public void shouldGetJustAMonth() throws Exception {
		Calendar date = addYear(1);
		date.set(MONTH, JANUARY);
		List<Integer> months = DateUtils.getMonthsOfPeriod(date.getTime(), date
				.getTime());
		assertNotNull(months);
		assertEquals(1, months.size());
		assertEquals(JANUARY, months.get(0).intValue());
	}

	@Test
	public void shouldGetJustMultipleMonths() throws Exception {
		Calendar dateBegin = addYear(1);
		dateBegin.set(DAY_OF_MONTH, 1);
		dateBegin.set(MONTH, FEBRUARY);
		Calendar dateEnd = addYear(1);
		dateBegin.set(DAY_OF_MONTH, 1);
		dateEnd.set(MONTH, MAY);
		List<Integer> months = DateUtils.getMonthsOfPeriod(dateBegin.getTime(),
				dateEnd.getTime());
		assertNotNull(months);
		assertEquals(4, months.size());
		assertEquals(Arrays.asList(FEBRUARY, MARCH, APRIL, MAY), months);

		dateBegin = addYear(1);
		dateBegin.set(MONTH, DECEMBER);
		dateEnd = addYear(2);
		dateEnd.set(MONTH, JANUARY);
		months = DateUtils.getMonthsOfPeriod(dateBegin.getTime(), dateEnd
				.getTime());
		assertNotNull(months);
		assertEquals(2, months.size());
		assertEquals(Arrays.asList(DECEMBER, JANUARY), months);

		dateBegin = addYear(1);
		dateBegin.set(MONTH, APRIL);
		dateEnd = addYear(3);
		dateEnd.set(MONTH, APRIL);
		months = DateUtils.getMonthsOfPeriod(dateBegin.getTime(), dateEnd
				.getTime());
		assertNotNull(months);
		assertEquals(12, months.size());
		assertEquals(Arrays.asList(APRIL, MAY, JUNE, JULY, AUGUST, SEPTEMBER, OCTOBER, NOVEMBER, DECEMBER, JANUARY, FEBRUARY, MARCH), months);
	}

	@Test
	public void shouldThrowExceptionOnInvalidPeriod() throws Exception {
		Calendar dateBegin = addYear(1);
		Calendar dateEnd = addYear(2);
		try {
			DateUtils.getMonthsOfPeriod(dateEnd.getTime(), dateBegin.getTime());
			fail("should throw exception on invalid period");
		} catch (IllegalArgumentException exception) {
			assertEquals("Invalid period", exception.getMessage());
		}
	}

	@Test
	public void shouldThrowExceptionOnPastDate() throws Exception {
		Date dateBegin = DateUtils.stringToDate("2009-02-01");
		Date dateEnd = DateUtils.stringToDate("2010-05-15");
		try {
			DateUtils.getMonthsOfPeriod(dateBegin, dateEnd);
			fail("should throw exception on past date");
		} catch (IllegalArgumentException exception) {
			assertEquals("Invalid date", exception.getMessage());
		}
	}

	@Test
	public void shouldTranslateIntMonthToString() {
		assertEquals("January", DateUtils.monthIntToString(0));
	}
	
	@Test
	public void todayShouldNotBePast() {
		Date today = DateUtils.stringToDate(daysFromTodayISO(0));
		assertTrue(DateUtils.isFutureDate(today));
	}
	
	@Test
	public void shouldValidateAllPastIssues(){
		Date dateBegin = null;
		Date dateEnd = null;		
		assertFalse(DateUtils.validateAll(dateBegin, dateEnd));
		
		dateEnd = DateUtils.stringToDate(daysFromTodayISO(1));
		assertFalse(DateUtils.validateAll(dateBegin, dateEnd));
		
		dateBegin = DateUtils.stringToDate(daysFromTodayISO(1));
		dateEnd = null;
		assertFalse(DateUtils.validateAll(dateBegin, dateEnd));
		
		dateBegin = DateUtils.stringToDate("2007-02-01");
		dateEnd = DateUtils.stringToDate(daysFromTodayISO(1));
		assertFalse(DateUtils.validateAll(dateBegin, dateEnd));
		
		dateBegin = DateUtils.stringToDate(daysFromTodayISO(1));
		dateEnd = DateUtils.stringToDate("2007-02-01");
		assertFalse(DateUtils.validateAll(dateBegin, dateEnd));
		
		dateBegin = DateUtils.stringToDate("2007-02-01");
		dateEnd = DateUtils.stringToDate("2007-03-01");
		assertFalse(DateUtils.validateAll(dateBegin, dateEnd));
		
		dateBegin = DateUtils.stringToDate(daysFromTodayISO(2));
		dateEnd = DateUtils.stringToDate(daysFromTodayISO(1));
		assertFalse(DateUtils.validateAll(dateBegin, dateEnd));
	}
}