package agileTraveler.weather;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.net.MalformedURLException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import agileTraveler.exceptions.CityNotFoundException;

public class WeatherHistoryParserTest {

	private static WeatherHistoryParser parser;
	private static String localURL;

	// Tests are based in London, United Kingdom
	// src/main/resources/UKXX0085.html

	@Before
	public void setUp() throws ClassNotFoundException, SQLException,
			CityNotFoundException {
		localURL = "file://" + System.getProperty("user.dir")
				+ "/src/main/resources/";
		parser = new WeatherHistoryParser("London");
		parser.setUrl(localURL);
	}

	@Test
	public void shouldCreateCorrectUrl() {
		assertEquals(localURL + "UKXX0085.html", parser.getUrl());
	}

	@Test
	public void shouldReturnListWithOneMonth() throws MalformedURLException,
			IOException {
		List<Integer> monthList = new ArrayList<Integer>();
		monthList.add(0);
		parser.setMonthList(monthList);
		assertTrue(parser.getWeatherHistories().size() > 0);
	}

	@Test
	public void shouldReturnExpectedMeanTemperature()
			throws MalformedURLException, IOException {
		List<Integer> monthList = new ArrayList<Integer>();
		monthList.add(0);
		parser.setMonthList(monthList);
		WeatherHistory weatherHistory = parser.getWeatherHistories().iterator()
				.next();
		assertEquals("41&deg;F", weatherHistory.getMeanTemperature());
	}

	@Test
	public void shouldReturnExpectedMaxTemperature()
			throws MalformedURLException, IOException {
		List<Integer> monthList = new ArrayList<Integer>();
		monthList.add(0);
		parser.setMonthList(monthList);
		WeatherHistory weatherHistory = parser.getWeatherHistories().iterator()
				.next();
		assertEquals("46&deg;F", weatherHistory.getMaxTemperature());
	} 

	@Test
	public void shouldReturnExpectedMinTemperature()
			throws MalformedURLException, IOException {
		List<Integer> monthList = new ArrayList<Integer>();
		monthList.add(0);
		parser.setMonthList(monthList);
		WeatherHistory weatherHistory = parser.getWeatherHistories().iterator()
				.next();
		assertEquals("36&deg;F", weatherHistory.getMinTemperature());
	}

	@Test
	public void shouldReturnTwoHistories() throws MalformedURLException,
			IOException {
		List<Integer> monthList = new ArrayList<Integer>();
		monthList.add(0);
		monthList.add(1);
		parser.setMonthList(monthList);
		assertEquals(2, parser.getWeatherHistories().size());
	}

	@Test
	public void shouldReturnThreeHistories() throws MalformedURLException,
			IOException {
		List<Integer> monthList = new ArrayList<Integer>();
		monthList.add(0);
		monthList.add(1);
		monthList.add(2);
		parser.setMonthList(monthList);
		assertEquals(3, parser.getWeatherHistories().size());
	}

	@Test
	public void shouldReturnTwoDifferentHistories()
			throws MalformedURLException, IOException {
		List<Integer> monthList = new ArrayList<Integer>();
		monthList.add(0);
		monthList.add(6);
		parser.setMonthList(monthList);
		List<WeatherHistory> weatherHistories = parser.getWeatherHistories();
		WeatherHistory weatherHistoryA = weatherHistories.get(0);
		WeatherHistory weatherHistoryB = weatherHistories.get(1);

		assertTrue(weatherHistoryA.getMeanTemperature() != weatherHistoryB
				.getMeanTemperature());
		assertTrue(weatherHistoryA.getMaxTemperature() != weatherHistoryB
				.getMaxTemperature());
		assertTrue(weatherHistoryA.getMinTemperature() != weatherHistoryB
				.getMinTemperature());
	}

	@Test
	public void shouldGetRomeNATemperature() throws Exception {
		parser = new WeatherHistoryParser("Rome");
		parser.setUrl(localURL);
		parser.setMonthList(Arrays.asList(Calendar.MARCH));
		String temperature = parser.getWeatherHistories().get(0)
				.getMeanTemperature();
		assertEquals("N/A&deg;F", temperature);
	}

	@Test
	public void shouldParsePindamonhangaba() throws CityNotFoundException,
			ClassNotFoundException, SQLException, MalformedURLException,
			IOException {
		parser = new WeatherHistoryParser("Pindamonhangaba");
		parser.setMonthList(Arrays.asList(Calendar.JUNE, Calendar.JULY));
		parser.setUrl(localURL);
		List<WeatherHistory> histories = parser.getWeatherHistories();
		assertNotNull(histories);
	}

	@Test
	public void shouldParseCitiesWithAccents() throws ClassNotFoundException,
			SQLException, CityNotFoundException, MalformedURLException,
			IOException {
		parser = new WeatherHistoryParser("São Paulo");
		parser.setMonthList(Arrays.asList(Calendar.MAY, Calendar.JUNE));
		parser.setUrl(localURL);
		assertEquals(2, parser.getWeatherHistories().size());
	}

	@Test
	public void shouldBeCaseInsensitive() throws ClassNotFoundException,
			SQLException, CityNotFoundException, MalformedURLException,
			IOException {
		parser = new WeatherHistoryParser("sao paulo");
		parser.setMonthList(Arrays.asList(Calendar.MAY, Calendar.JUNE));
		parser.setUrl(localURL);
		assertEquals(2, parser.getWeatherHistories().size());
	}
}