package agileTraveler.weather;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertNotNull;
import static junit.framework.Assert.assertTrue;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import org.jdom.JDOMException;
import org.junit.BeforeClass;
import org.junit.Test;

import agileTraveler.exceptions.CityNotFoundException;
import agileTraveler.model.Weather;

public class WeatherParserTest {

	private static String city;
	private static WeatherParser parser;

	@BeforeClass
	public static void initializeParser() throws Exception {
		city = "Belo Horizonte";
		parser = new WeatherParser(city);
	}

	@Test
	public void shouldHaveWeatherParser() throws Exception {
		assertNotNull(parser);
	}

	@Test
	public void shouldReturnWeatherFromRequestedCity() throws Exception {
		Weather weather = parser.getWeather();
		assertEquals(city, weather.getCity());
	}

	@Test(expected = CityNotFoundException.class)
	public void shouldThrowExceptionIfCityIsNotFound() throws Exception {
		String city = "This city does not exist";
		new WeatherParser(city);
	}

	@Test
	public void shouldHaveCurrentCondition() throws Exception {
		Weather weather = parser.getWeather();
		assertNotNull(weather.getCondition());
		// Google returned empty current_conditions.condition on 2010-05-18
		// 1:40pm
		assertTrue(weather.getCondition().length() >= 0);
	}

	@Test
	public void shouldHaveTemperature() throws Exception {
		WeatherParser parser = new WeatherParser("Manaus"); // Amazonas capital
		int manaus = parser.getWeather().getTemperature();

		parser = new WeatherParser("Juneau"); // Alaska capital
		int juneau = parser.getWeather().getTemperature();

		assertTrue("Amazonas (" + manaus + ") should be hotter than Alaska ("
				+ juneau + ")", manaus > juneau);
	}

	@Test
	public void shouldHaveHumidityPercentage() throws Exception {
		int humidity = parser.getWeather().getHumidity();
		assertTrue("Humidity was not percentage: " + humidity, 0 <= humidity
				&& humidity <= 100);
	}

	@Test
	public void shouldHaveIconUrl() throws Exception {
		Weather weather = parser.getWeather();
		assertNotNull(weather.getIconUrl());
		assertTrue(weather.getIconUrl().length() > 0);
	}

	@Test
	public void shouldHaveWindCondition() throws Exception {
		Weather weather = parser.getWeather();
		assertNotNull(weather.getWindCondition());
		assertTrue(weather.getWindCondition().length() > 0);
	}

	@Test
	public void shouldHaveForecastConditions() throws Exception {
		List<WeatherForecast> weatherForecasts = parser.getWeatherForecast();
		assertNotNull(weatherForecasts);
		assertFalse(weatherForecasts.isEmpty());
		WeatherForecast weatherForecast = weatherForecasts.get(0);
		assertNotNull(weatherForecast);
	}

	@Test
	public void shouldHaveForecastDayOfWeek() {
		List<WeatherForecast> weatherForecasts = parser.getWeatherForecast();
		List<String> daysOfWeek = Arrays.asList("Mon", "Tue", "Wed", "Thu",
				"Fri", "Sat", "Sun");
		String dayOfWeek;
		for (WeatherForecast weatherForecast : weatherForecasts) {
			dayOfWeek = weatherForecast.getDayOfWeek();
			assertNotNull(dayOfWeek);
			assertTrue(dayOfWeek + " is not day of week.", daysOfWeek
					.contains(dayOfWeek));
		}
	}

	@Test
	public void shouldHaveForecastLowAndHighTemperatures() {
		List<WeatherForecast> weatherForecasts = parser.getWeatherForecast();
		for (WeatherForecast weatherForecast : weatherForecasts) {
			int low = Integer.parseInt(weatherForecast.getLow());
			int high = Integer.parseInt(weatherForecast.getHigh());
			assertTrue("Highest temp is not greater then lowest", high > low);
		}
	}

	@Test
	public void shouldHaveForecastIconUrl() {
		List<WeatherForecast> weatherForecasts = parser.getWeatherForecast();
		for (WeatherForecast weatherForecast : weatherForecasts) {
			assertNotNull(weatherForecast.getIconUrl());
			assertTrue(weatherForecast.getIconUrl().length() > 0);
		}
	}

	@Test
	public void shouldHaveForecastCondition() {
		List<WeatherForecast> weatherForecasts = parser.getWeatherForecast();
		for (WeatherForecast weatherForecast : weatherForecasts) {
			assertNotNull(weatherForecast.getCondition());
			assertTrue(weatherForecast.getCondition().length() > 0);
		}
	}

	@Test
	public void shouldShowNAOnEmptyTemperatures() throws JDOMException,
			IOException, CityNotFoundException {
		parser = new WeatherParser("Tokyo", "numberlessForecast.xml");
		List<WeatherForecast> weatherForecasts = parser.getWeatherForecast();
		assertEquals("N/A", weatherForecasts.get(0).getHigh());
		assertEquals("N/A", weatherForecasts.get(0).getLow());
	}
}